/*-------------------------------------------------------------------------------------------*\
|                                                                                             |
|    Copyright (C) 1999 by Punk Productions Electronic Entertainment. All Rights Reserved.    | 
|                                                                                             |
|    Purpose.....: Shows how to load JPEG files into a DDsurface                              |
|    Programmer..: Nikolaus Brennig, 19, (Nikolaus.Brennig@nol.at)                            |
|	 Contact.....: virtalnik@nol.at || ICQ: 20345533										  |
|	 Internet....: http://cust.nol.at/ppee													  |
|    Help........: Joine the PPEE-GamesDev MailingList			                              |
|    Day created.: February 17th, 1999                                                        |
|    Update......: March, 30th, 1999                                                          |
|				   Can't remember anymore what i've changed :)								  |
|				   October, 3rd, 1999:														  |
|				   Completely rewritten, Faster Loading/Saving, C++ Interface                 |
|				   now the original JPEGLib 6.2 is used, also Support for                     |
|				   loading PPM/PGM files. Requires at least DirectX7                          |
|                                                                                             |
\*-------------------------------------------------------------------------------------------*/


#define WIN32_LEAN_AND_MEAN
#include "Main.h" 


//---------------------------------------------------------------------------------------------
// Do the whole intialisation stuff...
//---------------------------------------------------------------------------------------------
static BOOL InitPC( HINSTANCE hInstance, int nCmdShow )
{
	DDSURFACEDESC2      ddsd;
	DDSCAPS2            ddscaps;
	WNDCLASSEX          wcex;

	ZeroMemory(&wcex, sizeof(wcex));
	wcex.cbSize             = sizeof(wcex);
	wcex.style              = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc        = WindowProc;
	wcex.cbClsExtra         = 0;
	wcex.cbWndExtra         = 0;
	wcex.hInstance          = GetModuleHandle(NULL);
	wcex.hIcon              = LoadIcon(NULL, IDI_APPLICATION);
	wcex.hCursor            = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground      = HBRUSH(GetStockObject(BLACK_BRUSH));
	wcex.lpszMenuName       = NULL;
	wcex.lpszClassName      = TITLE;
	wcex.hIconSm            = NULL;
	if( !RegisterClassEx(&wcex) ) return FALSE;

	// create top-level window
	if( !(hwnd = CreateWindowEx(
		WS_EX_TOPMOST,
        TITLE,
        NAME,
        WS_POPUP,
        0, 
        0, 
        GetSystemMetrics(SM_CXSCREEN), 
        GetSystemMetrics(SM_CYSCREEN),
        NULL,
        NULL,
        GetModuleHandle(NULL),
        NULL
	)) ) return FALSE;

    // Yeah, show the window...
    ShowWindow( hwnd, SW_NORMAL ); 
    UpdateWindow( hwnd ); 
    SetFocus( hwnd );

    // Create the DirectDraw7object...
    if( FAILED(DirectDrawCreateEx(NULL, (VOID**)&lpDD, IID_IDirectDraw7, NULL)) )
		return Error( "DDObject7 creation failed! DirectX7 is required!" );

    // Now lets set the cooperativelevel of our application...
    if( FAILED(lpDD->SetCooperativeLevel( hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_NOWINDOWCHANGES )) )
		return Error( "SetCooperativeLevel failed!" );

    // Now lets set the Displaymode: 640x480 in 32 bit Mode...
    if( FAILED(lpDD->SetDisplayMode( 640, 480, 32, 0, 0 )) )
		return Error( "SetDisplayMode failed!" );

    // Now we create the primary surface. 
	ZeroMemory( &ddsd, sizeof(ddsd) );
    ddsd.dwSize            = sizeof( ddsd );
    ddsd.dwFlags           = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
    ddsd.dwBackBufferCount = 2;
    if( FAILED(lpDD->CreateSurface( &ddsd, &lpDDSPrimary, NULL )) )
		return Error( "PrimarySurface creation failed!");

    // Create the Backbuffer. To this surface the whole graphics stuff is drawn.
	ZeroMemory( &ddscaps, sizeof(ddscaps) );
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
    if( FAILED(lpDDSPrimary->GetAttachedSurface(&ddscaps, &lpDDSBack)) )
		return Error( "Couldn't attatch the Backbuffer!" );

    // Now let's load our TestImages...
	GetImage( "Test JPG normal.jpg", &lpDDSJPGNormal );
	GetImage( "Test JPG progressive.jpg", &lpDDSJPGProgressive );
	GetImage( "Test JPG GrayScale.jpg",	&lpDDSJPGGrayScale );
	GetImage( "Test PPM Normal.ppm", &lpDDSPPMNormal );
	GetImage( "Test PGM Normal.pgm", &lpDDSPGMNormal );

    // Ok, start the whole thing...
    ProgramState = PS_INITAPP;

	// Yeah...
	return TRUE;
}


//---------------------------------------------------------------------------------------------
// This function displays an Errormessage and terminates the app...
//---------------------------------------------------------------------------------------------
BOOL Error( char *err )
{
	MessageBox( hwnd, err, TITLE, MB_ICONHAND+MB_OK );
	DestroyWindow ( hwnd );
  
	return FALSE;
}


//---------------------------------------------------------------------------------------------
// WinMain:
//---------------------------------------------------------------------------------------------
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG         msg;
    
    lpCmdLine     = lpCmdLine;
    hPrevInstance = hPrevInstance;

	// Init our stuff...
    if( !InitPC( hInstance, nCmdShow ) ) 
		return Error("PC Initialization failed!");

	// MainMessageLoop...
    while( 1 )
    {
		if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
		{
			if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
			TranslateMessage(&msg); 
			DispatchMessage(&msg);
		}
		else if( bActive )
		{
			// App is active --> do the main procedure...
			HeartBeat();
		}
		else
		{
			// Ok, nothing to do, so do nothing :-))
			WaitMessage();
		}
    }
}


